@@ -1,5 +1,5 @@
 /*
- * Copyright © 2009 Christian Persch <chpe@gnome.org>
+ * Copyright © 2009, 2010 Christian Persch <chpe@gnome.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,10 +20,15 @@
 #include "ar-style.h"
 #include "ar-style-private.h"
 
+#include <libgames-support/games-debug.h>
+#include <libgames-support/games-glib-compat.h>
+
 enum
 {
   PROP_0,
+  PROP_CARD_OVERHANG,
   PROP_CARD_SLOT_RATIO,
+  PROP_CARD_STEP,
   PROP_CARD_THEME,
   PROP_CLICK_TO_MOVE,
   PROP_DND_DRAG_THRESHOLD,
@@ -33,9 +38,14 @@ enum
   PROP_FOCUS_LINE_WIDTH,
   PROP_FOCUS_PADDING,
   PROP_INTERIOR_FOCUS,
+#ifndef HAVE_CLUTTER
+  PROP_PIXBUF_DRAWING,
+#endif
   PROP_RTL,
   PROP_SELECTION_COLOR,
-  PROP_TOUCHSCREEN_MODE,
+  PROP_SHOW_TOOLTIPS,
+  PROP_SHOW_STATUS_MESSAGES,
+  PROP_TOUCHSCREEN_MODE
 };
 
 /* private functions */
@@ -51,8 +61,15 @@ ar_style_init (ArStyle *style)
 
   priv = style->priv = G_TYPE_INSTANCE_GET_PRIVATE (style, AR_TYPE_STYLE, ArStylePrivate);
 
+#ifdef HAVE_CLUTTER
   _ar_clutter_color_from_gdk_color (&priv->selection_color, &default_selection_color);
+#else
+  priv->selection_color = default_selection_color;
+#endif
+
   priv->card_slot_ratio = DEFAULT_CARD_SLOT_RATIO;
+  priv->card_overhang = DEFAULT_CARD_OVERHANG;
+  priv->card_step = DEFAULT_CARD_STEP;
   priv->dnd_drag_threshold = 8;
   priv->double_click_time = 250;
   priv->focus_line_width = 1;
@@ -65,6 +82,29 @@ ar_style_init (ArStyle *style)
   priv->rtl = FALSE;
   priv->interior_focus = FALSE;
   priv->click_to_move = FALSE;
+  priv->enable_tooltips = DEFAULT_SHOW_TOOLTIPS;
+  priv->enable_status_messages = DEFAULT_SHOW_STATUS_MESSAGES;
+
+#ifndef HAVE_CLUTTER
+
+#ifdef HAVE_HILDON
+  priv->pixbuf_drawing = FALSE;
+#else
+{
+  const char *env;
+
+  env = g_getenv ("AISLERIOT_PIXBUF_DRAWING");
+
+  /* Default to pixbuf drawing */
+  priv->pixbuf_drawing = env == NULL || g_ascii_strtoull (env, NULL, 10) != 0;
+}
+#endif /* HAVE_HILDON */
+
+  _games_debug_print (GAMES_DEBUG_GAME_STYLE,
+                      "[ArStyle %p] Using %s drawing\n",
+                      style, priv->pixbuf_drawing ? "pixbuf" : "pixmap");
+
+#endif /* !HAVE_CLUTTER */
 }
 
 static void
@@ -90,10 +130,18 @@ ar_style_get_property (GObject    *object,
   ArStylePrivate *priv = style->priv;
 
   switch (property_id) {
+    case PROP_CARD_OVERHANG:
+      g_value_set_double (value, ar_style_get_card_overhang (style));
+      break;
+
     case PROP_CARD_SLOT_RATIO:
       g_value_set_double (value, ar_style_get_card_slot_ratio (style));
       break;
 
+    case PROP_CARD_STEP:
+      g_value_set_double (value, ar_style_get_card_step (style));
+      break;
+
     case PROP_CARD_THEME:
       g_value_set_object (value, ar_style_get_card_theme (style));
       break;
@@ -130,6 +178,12 @@ ar_style_get_property (GObject    *object,
       g_value_set_boolean (value, ar_style_get_interior_focus (style));
       break;
 
+#ifndef HAVE_CLUTTER
+    case PROP_PIXBUF_DRAWING:
+      g_value_set_boolean (value, ar_style_get_pixbuf_drawing (style));
+      break;
+#endif
+
     case PROP_RTL:
       g_value_set_boolean (value, ar_style_get_rtl (style));
       break;
@@ -138,6 +192,14 @@ ar_style_get_property (GObject    *object,
       g_value_set_boxed (value, &priv->selection_color);
       break;
 
+    case PROP_SHOW_TOOLTIPS:
+      g_value_set_boolean (value, ar_style_get_show_tooltips (style));
+      break;
+
+    case PROP_SHOW_STATUS_MESSAGES:
+      g_value_set_boolean (value, ar_style_get_show_status_messages (style));
+      break;
+
     case PROP_TOUCHSCREEN_MODE:
       g_value_set_boolean (value, ar_style_get_touchscreen_mode (style));
       break;
@@ -154,8 +216,21 @@ ar_style_set_property (GObject      *object,
                        GParamSpec   *pspec)
 {
   ArStyle *style = AR_STYLE (object);
+  ArStylePrivate *priv = style->priv;
 
   switch (property_id) {
+    case PROP_CARD_OVERHANG:
+      priv->card_overhang = g_value_get_double (value);
+      break;
+
+    case PROP_CARD_SLOT_RATIO:
+      priv->card_slot_ratio = g_value_get_double (value);
+      break;
+
+    case PROP_CARD_STEP:
+      priv->card_step = g_value_get_double (value);
+      break;
+
     case PROP_CARD_THEME:
       ar_style_set_card_theme (style, g_value_get_object (value));
       break;
@@ -164,24 +239,75 @@ ar_style_set_property (GObject      *object,
       ar_style_set_click_to_move (style, g_value_get_boolean (value));
       break;
 
+    case PROP_DND_DRAG_THRESHOLD:
+      priv->dnd_drag_threshold = g_value_get_int (value);
+      break;
+
+    case PROP_DOUBLE_CLICK_TIME:
+      priv->double_click_time = g_value_get_int (value);
+      break;
+
     case PROP_ENABLE_ANIMATIONS:
       ar_style_set_enable_animations (style, g_value_get_boolean (value));
       break;
 
+    case PROP_FOCUS_LINE_WIDTH:
+      priv->focus_line_width = g_value_get_int (value);
+      break;
+
+    case PROP_FOCUS_PADDING:
+      priv->focus_padding = g_value_get_int (value);
+      break;
+
+    case PROP_INTERIOR_FOCUS:
+      priv->interior_focus = g_value_get_boolean (value) != FALSE;
+      break;
+
+#ifndef HAVE_CLUTTER
+    case PROP_PIXBUF_DRAWING:
+      priv->pixbuf_drawing = g_value_get_boolean (value) != FALSE;
+#endif
+
+    case PROP_RTL:
+      priv->rtl = g_value_get_boolean (value) != FALSE;
+      break;
+
+    case PROP_SELECTION_COLOR: {
+#ifdef HAVE_CLUTTER
+      ClutterColor *color;
+
+      if ((color = g_value_get_boxed (value)) != NULL) {
+        priv->selection_color = *color;
+      } else {
+        _ar_clutter_color_from_gdk_color (&priv->selection_color, &default_selection_color);
+      }
+#else
+      GdkColor *color;
+
+      if ((color = g_value_get_boxed (value)) != NULL) {
+        priv->selection_color = *color;
+      } else {
+        priv->selection_color = default_selection_color;
+      }
+#endif
+      break;
+    }
+
     case PROP_ENABLE_SOUND:
       ar_style_set_enable_sound (style, g_value_get_boolean (value));
       break;
 
-    case PROP_CARD_SLOT_RATIO:
-    case PROP_DND_DRAG_THRESHOLD:
-    case PROP_DOUBLE_CLICK_TIME:
-    case PROP_FOCUS_LINE_WIDTH:
-    case PROP_FOCUS_PADDING:
-    case PROP_INTERIOR_FOCUS:
-    case PROP_RTL:
-    case PROP_SELECTION_COLOR:
+    case PROP_SHOW_TOOLTIPS:
+      priv->enable_tooltips = g_value_get_boolean (value) != FALSE;
+      break;
+
+    case PROP_SHOW_STATUS_MESSAGES:
+      priv->enable_status_messages = g_value_get_boolean (value) != FALSE;
+      break;
+
     case PROP_TOUCHSCREEN_MODE:
-      /* not writable */
+      priv->touchscreen_mode = g_value_get_boolean (value) != FALSE;
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -192,7 +318,9 @@ static void
 ar_style_class_init (ArStyleClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+#ifdef HAVE_CLUTTER
   ClutterColor color;
+#endif
 
   g_type_class_add_private (klass, sizeof (ArStylePrivate));
 
@@ -205,7 +333,35 @@ ar_style_class_init (ArStyleClass *klass)
      PROP_CARD_SLOT_RATIO,
      g_param_spec_double (AR_STYLE_PROP_CARD_SLOT_RATIO, NULL, NULL,
                           0.1, 1.0, DEFAULT_CARD_SLOT_RATIO,
-                          G_PARAM_READABLE |
+                          G_PARAM_READWRITE |
+                          G_PARAM_STATIC_STRINGS));
+
+  /**
+   * ArStyle:card-overhang:
+   *
+   * This controls how much of a card is allowed to hang off of the bottom
+   * of the screen. If set to %0.0, the last card is always fully visible.
+   */
+  g_object_class_install_property
+    (object_class,
+     PROP_CARD_OVERHANG,
+     g_param_spec_double (AR_STYLE_PROP_CARD_OVERHANG, NULL, NULL,
+                          0.0, 1.0, DEFAULT_CARD_OVERHANG,
+                          G_PARAM_READWRITE |
+                          G_PARAM_STATIC_STRINGS));
+
+  /**
+   * ArStyle:card-step:
+   *
+   * This controls how much one card is offset the previous one in card stacks.
+   * A game-specified a value for the card step takes precedence over this.
+   */
+  g_object_class_install_property
+    (object_class,
+     PROP_CARD_STEP,
+     g_param_spec_double (AR_STYLE_PROP_CARD_STEP, NULL, NULL,
+                          MIN_CARD_STEP, MAX_CARD_STEP, DEFAULT_CARD_STEP,
+                          G_PARAM_READWRITE |
                           G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
@@ -229,7 +385,7 @@ ar_style_class_init (ArStyleClass *klass)
      PROP_DND_DRAG_THRESHOLD,
      g_param_spec_int (AR_STYLE_PROP_DND_DRAG_THRESHOLD, NULL, NULL,
                        1, G_MAXINT, 8,
-                       G_PARAM_READABLE |
+                       G_PARAM_READWRITE |
                        G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
@@ -237,7 +393,7 @@ ar_style_class_init (ArStyleClass *klass)
      PROP_DOUBLE_CLICK_TIME,
      g_param_spec_int (AR_STYLE_PROP_DOUBLE_CLICK_TIME, NULL, NULL,
                        0, G_MAXINT, 250,
-                       G_PARAM_READABLE |
+                       G_PARAM_READWRITE |
                        G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
@@ -261,7 +417,7 @@ ar_style_class_init (ArStyleClass *klass)
      PROP_FOCUS_LINE_WIDTH,
      g_param_spec_int (AR_STYLE_PROP_FOCUS_LINE_WIDTH, NULL, NULL,
                        0, G_MAXINT, 1,
-                       G_PARAM_READABLE |
+                       G_PARAM_READWRITE |
                        G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
@@ -269,7 +425,7 @@ ar_style_class_init (ArStyleClass *klass)
      PROP_FOCUS_PADDING,
      g_param_spec_int (AR_STYLE_PROP_FOCUS_PADDING, NULL, NULL,
                        0, G_MAXINT, 1,
-                       G_PARAM_READABLE |
+                       G_PARAM_READWRITE |
                        G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
@@ -277,37 +433,85 @@ ar_style_class_init (ArStyleClass *klass)
      PROP_INTERIOR_FOCUS,
      g_param_spec_boolean (AR_STYLE_PROP_INTERIOR_FOCUS, NULL, NULL,
                            FALSE,
-                           G_PARAM_READABLE |
+                           G_PARAM_READWRITE |
                            G_PARAM_STATIC_STRINGS));
 
+#ifndef HAVE_CLUTTER
+  g_object_class_install_property
+    (object_class,
+     PROP_PIXBUF_DRAWING,
+     g_param_spec_boolean (AR_STYLE_PROP_PIXBUF_DRAWING, NULL, NULL,
+                           DEFAULT_PIXBUF_DRAWING,
+                           G_PARAM_READWRITE |
+                           G_PARAM_STATIC_STRINGS));
+#endif /* !HAVE_CLUTTER */
+
   g_object_class_install_property
     (object_class,
      PROP_RTL,
      g_param_spec_boolean (AR_STYLE_PROP_RTL, NULL, NULL,
                            FALSE,
-                           G_PARAM_READABLE |
+                           G_PARAM_READWRITE |
                            G_PARAM_STATIC_STRINGS));
 
+#ifdef HAVE_CLUTTER
   _ar_clutter_color_from_gdk_color (&color, &default_selection_color);
   g_object_class_install_property
     (object_class,
      PROP_SELECTION_COLOR,
      clutter_param_spec_color (AR_STYLE_PROP_SELECTION_COLOR, NULL, NULL,
                                &color,
-                               G_PARAM_READABLE |
+                               G_PARAM_READWRITE |
                                G_PARAM_STATIC_STRINGS));
+#else
+  g_object_class_install_property
+    (object_class,
+     PROP_SELECTION_COLOR,
+     g_param_spec_boxed (AR_STYLE_PROP_SELECTION_COLOR, NULL, NULL,
+                         GDK_TYPE_COLOR,
+                         G_PARAM_READWRITE |
+                         G_PARAM_STATIC_STRINGS));
+#endif /* HAVE_CLUTTER */
+
+  /**
+   * ArStyle:show-tooltips:
+   *
+   * Whether to show tooltips on the cards and slots.
+   */
+  g_object_class_install_property
+    (object_class,
+     PROP_SHOW_TOOLTIPS,
+     g_param_spec_boolean (AR_STYLE_PROP_SHOW_TOOLTIPS, NULL, NULL,
+                           DEFAULT_SHOW_TOOLTIPS,
+                           G_PARAM_READWRITE |
+                           G_PARAM_STATIC_STRINGS));
+
+  /**
+   * ArStyle:show-status-messages:
+   *
+   * Whether to show status messages on motion over the cards and slots.
+   */
+  g_object_class_install_property
+    (object_class,
+     PROP_SHOW_STATUS_MESSAGES,
+     g_param_spec_boolean (AR_STYLE_PROP_SHOW_STATUS_MESSAGES, NULL, NULL,
+                           DEFAULT_SHOW_STATUS_MESSAGES,
+                           G_PARAM_READWRITE |
+                           G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (object_class,
      PROP_TOUCHSCREEN_MODE,
      g_param_spec_boolean (AR_STYLE_PROP_TOUCHSCREEN_MODE, NULL, NULL,
                            FALSE,
-                           G_PARAM_READABLE |
+                           G_PARAM_READWRITE |
                            G_PARAM_STATIC_STRINGS));
 }
 
 /* private API */
 
+#ifdef HAVE_CLUTTER
+
 void
 _ar_clutter_color_from_gdk_color (ClutterColor *clutter_color,
                                   const GdkColor *gdk_color)
@@ -318,6 +522,8 @@ _ar_clutter_color_from_gdk_color (ClutterColor *clutter_color,
   clutter_color->alpha = 0xff;
 }
 
+#endif /* HAVE_CLUTTER */
+
 /* public API */
 
 /**
@@ -506,7 +712,6 @@ ar_style_get_interior_focus (ArStyle *style)
   return priv->interior_focus;
 }
 
-
 /**
  * ar_style_get_rtl:
  * @style: an #ArStyle
@@ -521,6 +726,34 @@ ar_style_get_rtl (ArStyle *style)
   return priv->rtl;
 }
 
+/**
+ * ar_style_get_show_tooltips:
+ * @style: an #ArStyle
+ *
+ * Returns:
+ */
+gboolean
+ar_style_get_show_tooltips (ArStyle *style)
+{
+  ArStylePrivate *priv = style->priv;
+
+  return priv->enable_tooltips;
+}
+
+/**
+ * ar_style_get_show_status_messages:
+ * @style: an #ArStyle
+ *
+ * Returns:
+ */
+gboolean
+ar_style_get_show_status_messages (ArStyle *style)
+{
+  ArStylePrivate *priv = style->priv;
+
+  return priv->enable_status_messages;
+}
+
 /**
  * ar_style_get_double_click_time:
  * @style: an #ArStyle
@@ -576,6 +809,34 @@ ar_style_get_card_slot_ratio (ArStyle *style)
   return priv->card_slot_ratio;
 }
 
+/**
+ * ar_style_get_card_overhang:
+ * @style: an #ArStyle
+ *
+ * Returns:
+ */
+double
+ar_style_get_card_overhang (ArStyle *style)
+{
+  ArStylePrivate *priv = style->priv;
+
+  return priv->card_overhang;
+}
+
+/**
+ * ar_style_get_card_step:
+ * @style: an #ArStyle
+ *
+ * Returns:
+ */
+double
+ar_style_get_card_step (ArStyle *style)
+{
+  ArStylePrivate *priv = style->priv;
+
+  return priv->card_step;
+}
+
 /**
  * ar_style_get_selection_color:
  * @style: an #ArStyle
@@ -584,7 +845,11 @@ ar_style_get_card_slot_ratio (ArStyle *style)
  */
 void
 ar_style_get_selection_color (ArStyle *style,
+#ifdef HAVE_CLUTTER
                               ClutterColor * const color)
+#else
+                              GdkColor * const color)
+#endif
 {
   ArStylePrivate *priv = style->priv;
 
@@ -619,3 +884,20 @@ ar_style_check_dnd_drag_threshold (ArStyle *style,
   return (ABS (x2 - x1) > priv->dnd_drag_threshold ||
           ABS (y2 - y1) > priv->dnd_drag_threshold);
 }
+
+#ifndef HAVE_CLUTTER
+/**
+ * ar_style_get_pixbuf_drawing:
+ * @style:
+ *
+ * Returns: wether to use pixbuf drawing
+ */
+gboolean
+ar_style_get_pixbuf_drawing (ArStyle *style)
+{
+  ArStylePrivate *priv = style->priv;
+
+  return priv->pixbuf_drawing;
+}
+
+#endif /* !HAVE_CLUTTER */
